#include "SDL.h"

#include <stdio.h>
#include <stdbool.h>

#include "fcgame.h"

void add_updaterect(unsigned long x, unsigned long y){
	if(!updateall){
		//don't have doubles
		for(int i = 0; i < numupdaterects; i++){
			if(updaterects[i].x == x && updaterects[i].y == y){
				return;
			}
		}
		
		updaterects[numupdaterects].x = x*TILEWIDTH-screenx;
		updaterects[numupdaterects].y = y*TILEHEIGHT-screeny;
		updaterects[numupdaterects].w = TILEWIDTH;
		updaterects[numupdaterects].h = TILEHEIGHT;
		numupdaterects++;
		if(numupdaterects >= 99){
			updateall = true;
			numupdaterects = 0;
		}
	}
}

void do_print_sprite(int number, unsigned long x, unsigned long y){
	SDL_Rect srcrect = {(((number%13)*TILEWIDTH)), (((number/13)*TILEHEIGHT)), TILEWIDTH, TILEHEIGHT};
	SDL_Rect destrect = {x, y, TILEWIDTH, TILEHEIGHT};
	
	if(number == 0){
		return;
	}
	
	// Blit onto the screen surface
	if(SDL_BlitSurface(tileset, &srcrect, screen, &destrect) < 0){
		fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());
	}
}

void print_sprite(int number, signed long x, signed long y){
	SDL_Rect srcrect = {(((number%13)*TILEWIDTH)), (((number/13)*TILEHEIGHT)), TILEWIDTH, TILEHEIGHT};
	SDL_Rect destrect = {x-bufferx, y-buffery, TILEWIDTH, TILEHEIGHT};
	
	if(!checkbounds(stupiddiv(x),stupiddiv(y))){
		SDL_FillRect(sbuffer, &destrect, SDL_MapRGB(screen->format, 0, 0, 0));
		return;
	}
	if(number == 0){
		return;
	}
	
	// Blit onto the screen surface
	if(SDL_BlitSurface(tileset, &srcrect, sbuffer, &destrect) < 0){
		fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());
	}
}

void do_clear_tile(unsigned short x, unsigned short y){
	SDL_Rect blankrect;
	blankrect.x = x*TILEWIDTH;
	blankrect.y = y*TILEHEIGHT;
	blankrect.w = TILEWIDTH;
	blankrect.h = TILEHEIGHT;
	SDL_FillRect(screen, &blankrect, SDL_MapRGB(screen->format, 255, 255, 255));
}

void clear_tile(unsigned short x, unsigned short y){
	SDL_Rect blankrect;
	blankrect.x = x*TILEWIDTH-bufferx;
	blankrect.y = y*TILEHEIGHT-buffery;
	blankrect.w = TILEWIDTH;
	blankrect.h = TILEHEIGHT;
	SDL_FillRect(sbuffer, &blankrect, SDL_MapRGB(screen->format, 0, 0, 0));
}

void grey_tile(unsigned short x, unsigned short y){
	printf("grey_tile(): This function is old, don't use it\n");
	SDL_Rect blankrect;
	blankrect.x = x*TILEWIDTH-bufferx;
	blankrect.y = y*TILEHEIGHT-buffery;
	blankrect.w = TILEWIDTH;
	blankrect.h = TILEHEIGHT;
	SDL_FillRect(screen, &blankrect, SDL_MapRGB(screen->format, 100, 100, 100));
}

size_2d draw_entity(entity_t *entity){//warning segfaults
	unsigned short oldx = entity->x;
	unsigned short oldy = entity->y;
	
	if(entity->realx < (entity->x)*TILEWIDTH){
		oldx--;
	}else if(entity->realx > (entity->x)*TILEHEIGHT){
		oldx++;
	}else if(entity->realy < (entity->y)*TILEWIDTH){
		oldy--;
	}else if(entity->realy > (entity->y)*TILEHEIGHT){
		oldy++;
	}
	add_updaterect(oldx,oldy);
	
	clear_tile(entity->x, entity->y);
	clear_tile(oldx, oldy);
	print_sprite(layer_bottom[((oldy*mapwidth)+oldx)], (oldx)*TILEWIDTH, (oldy)*TILEHEIGHT);
	print_sprite(layer_bottom[(((entity->y)*mapwidth)+(entity->x))], (entity->x)*TILEWIDTH, (entity->y)*TILEHEIGHT);
	
	unsigned short tilenum = entity->tileid + entity->direction;
	
	if(entity->step/TICK_PER_STEP == 0 || entity->step/TICK_PER_STEP == 2){
		tilenum+=4;
	}else if(entity->step/TICK_PER_STEP == 1){
		tilenum+=8;
	}
	
	print_sprite(tilenum, entity->realx, entity->realy);
	print_sprite(layer_top[(entity->y)+1], (entity->x)*TILEWIDTH, ((entity->y)-1)*TILEHEIGHT);
	
	size_2d return2d = {oldx, oldy};
	return return2d;
}

void draw_tile(signed short x, signed short y){
	if(frameskip) return;
	
	if(!checkbounds(x,y)){
		clear_tile(x,y);
		//SDL_FillRect(screen, &destrect, SDL_MapRGB(screen->format, 0, 0, 0));
		return;
	}
	
	clear_tile(x, y);
	print_sprite(get_tileid(layer_bottom, x, y), x*TILEWIDTH, y*TILEHEIGHT);
	
	if(get_tileid(layer_interact, x, y)){
		print_sprite(get_tileid(layer_interact, x, y), x*TILEWIDTH, y*TILEHEIGHT);
	}else if(get_entity(x,y) != (entity_t *)NULL){
		//////here the handling is difficult///////
		entity_t *entity = get_entity(x,y);
		
		if(entity->realx != (entity->x)*TILEWIDTH || entity->realy != (entity->y)*TILEHEIGHT){
			draw_entity(entity);//also high segfault risk
		}else{
			print_sprite(entity->tileid+entity->direction+4, x*TILEWIDTH, y*TILEHEIGHT);
		}
	}
	
	print_sprite(get_tileid(layer_top, x, y), x*TILEWIDTH, y*TILEHEIGHT);
	
	add_updaterect(x,y);
}

void drawscreen(){
	frameskip = false;
	for(signed short y = stupiddiv(buffery); y <= stupiddiv(buffery)+((sbuffer->h)/TILEHEIGHT); y++){
		for(signed short x = stupiddiv(bufferx); x <= stupiddiv(bufferx)+((sbuffer->w)/TILEWIDTH); x++){
			draw_tile(x,y);
		}
	}
}

void drawentities(){
	for(signed short y = stupiddiv(buffery); y <= stupiddiv(buffery)+((sbuffer->h)/TILEHEIGHT); y++){
		for(signed short x = stupiddiv(bufferx); x <= stupiddiv(bufferx)+((sbuffer->w)/TILEWIDTH); x++){
			if(!checkbounds(x,y)){
				//clear_tile(x,y);
				continue;
			}
			if(get_entity(x,y) != (entity_t *)NULL && !get_tileid(layer_interact, x, y)){
				grey_tile(x,y);
				draw_entity(get_entity(x,y));
			}
		}
	}
}

void refresh_entities(){
	while(numneeddraw){
		draw_tile(needdrawx[numneeddraw],needdrawy[numneeddraw]);
		numneeddraw--;
	}
	
	for(signed short y = stupiddiv(buffery); y <= stupiddiv(buffery)+((sbuffer->h)/TILEHEIGHT); y++){
		for(signed short x = stupiddiv(bufferx); x <= stupiddiv(bufferx)+((sbuffer->w)/TILEWIDTH); x++){
			if(!checkbounds(x,y)){
				//clear_tile(x,y);
				continue;
			}
			if(get_entity(x,y) != (entity_t *)NULL && !get_tileid(layer_interact, x, y)){
				entity_t *entity = get_entity(x,y);
				
				entity->step++;
				if(entity->step/TICK_PER_STEP == 4){
					entity->step = 0;
				}
				if(entity->realx != (entity->x)*TILEWIDTH){
					unsigned short oldx = x;
					if(entity->realx < (entity->x)*TILEWIDTH){
						entity->realx++;
						oldx--;
					}else if(entity->realx > (entity->x)*TILEWIDTH){
						entity->realx--;
						oldx++;
					}
					draw_tile(x,y);
					if(entity->realx == (entity->x)*TILEWIDTH){
						numneeddraw++;
						needdrawx[numneeddraw] = oldx;
						needdrawy[numneeddraw] = y;
					}
				}else if(entity->realy != (entity->y)*TILEHEIGHT){
					unsigned short oldy = y;
					if(entity->realy < (entity->y)*TILEHEIGHT){
						entity->realy++;
						oldy--;
					}else if(entity->realy > (entity->y)*TILEHEIGHT){
						entity->realy--;
						oldy++;
					}
					draw_tile(x,y);
					if(entity->realy == (entity->y)*TILEHEIGHT){
						numneeddraw++;
						needdrawx[numneeddraw] = x;
						needdrawy[numneeddraw] = oldy;
					}
				}
			}
		}
	}
}

void move_screen(signed long newx, signed long newy){
	if(newx == screenx && newy == screeny){
		return;
	}
	
	screenx = newx;
	screeny = newy;
}

inline bool space_free(signed short x, signed short y){
	return
	x >= 0 &&
	x < mapwidth &&
	y >= 0 &&
	y < mapheight &&
	!get_tileid(layer_interact, x, y) &&
	get_entity(x, y) == NULL
	;
}

bool entity_move(entity_t *entity, signed short xoffset, signed short yoffset){
	if(!xoffset && !yoffset){
		return false;
	}
	
	//printf("x: %i, y: %i\n", xoffset, yoffset);
	if(entity->realx == entity->x*TILEWIDTH && entity->realy == entity->y*TILEHEIGHT){
		unsigned short oldx = entity->x;
		unsigned short oldy = entity->y;
		
		signed short newx = entity->x;
		signed short newy = entity->y;
		
		if(xoffset){
			newx += xoffset;
		}else if(yoffset){
			newy += yoffset;
		}
		
		
		if(space_free(newx, newy)){
			entity->x = newx;
			entity->y = newy;
		}
		
		
		if(xoffset > 0){//right
			entity->direction = 1;
		}else if(xoffset < 0){//left
			entity->direction = 3;
		}else if(yoffset > 0){//down
			entity->direction = 2;
		}else if(yoffset < 0){//up
			entity->direction = 0;
		}
		
		layer_entities[((oldy*mapwidth)+oldx)] = NULL;
		layer_entities[((entity->y*mapwidth)+entity->x)] = entity;
		
		draw_tile(oldx,oldy);
		
		if(!space_free(((signed short)oldx) + xoffset, ((signed short)oldy) + yoffset)){
			return false;
		}
		
		draw_tile(entity->x,entity->y);
	}
	
	return true;  //true, for now
}
