#include <stdbool.h>

#include "SDL.h"
#include "lua.h"

////////////////variables///////////////////

SDL_Surface *screen;
SDL_Surface *sbuffer;
SDL_Surface *font;
SDL_Surface *highfont;
SDL_Surface *tileset;

SDL_Rect updaterects[100];
extern int numupdaterects;

//only entities move
typedef struct{
	char name[16];
	unsigned short tileid;//START tile
	unsigned char direction;
	unsigned char step;
	unsigned short x,y;	//for fast access
	unsigned long realx, realy;	//value in pixels
} entity_t;

typedef struct{
	unsigned short x,y;
} size_2d;

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	#define rmask 0xff000000
	#define gmask 0x00ff0000
	#define bmask 0x0000ff00
	#define amask 0x000000ff
#else
	#define rmask 0x000000ff
	#define gmask 0x0000ff00
	#define bmask 0x00ff0000
	#define amask 0xff000000
#endif


////three layers///
unsigned short *layer_bottom;
unsigned short *layer_interact;
entity_t **layer_entities;
unsigned short *layer_top;

unsigned char *bufferneedsdraw;

unsigned short needdrawx[50];//bit of a hack
unsigned short needdrawy[50];//values which need redrawing, but the engine cannot easily detect it

char current_map[17];

extern unsigned char whatplayer;
extern int screenwidth, screenheight;
extern signed long screenx;
extern signed long screeny;
extern signed long bufferx;
extern signed long buffery;
extern unsigned char numneeddraw;
extern unsigned short mapwidth;	//define these in the map file later
extern unsigned short mapheight;
extern bool quit;
extern bool frameskip;
extern bool updateall;

extern entity_t *player;

#define TICK_INTERVAL	15
#define TICK_PER_STEP 9

#define character_start 78
#define players_tile_start (character_start + whatplayer*13)


#define TILEWIDTH 16
#define TILEHEIGHT 16


lua_State* luaVM;


//////////////////functions//////////////
//main.c
void gameloop();

//draw.c
void do_print_sprite(int number, unsigned long x, unsigned long y);
void print_sprite(int number, signed long x, signed long y);
void clear_tile(unsigned short x, unsigned short y);
void do_clear_tile(unsigned short x, unsigned short y);
void draw_tile(signed short x, signed short y);
void drawscreen();
void drawentities();
void refresh_entities();
void move_screen(signed long newx, signed long newy);
bool entity_move(entity_t *entity, signed short xoffset, signed short yoffset);

//update.c
void update();
void do_updatebuffer();
void do_update();

//util.c
void delay();
void calculate_FPS();
void init_time();
signed short stupiddiv(signed short value);
bool checkbounds(signed short x, signed short y);
void clear_input();
SDL_Surface *image_load(char *filename);

//map.c
unsigned short get_tileid(unsigned short *layer, signed short x, signed short y);
entity_t *get_entity(signed short x, signed short y);
void entity_face(entity_t *entity, int direction);
bool loadmap(char *filename);
void unloadmap();
int create_entity(lua_State* luaS);
int setup_player(lua_State* luaS);
void reload_entities();
void free_entities();

//sound.c
void init_audio();
void set_bgmusic(char *file);
void load_sounds();
void beep();

//lua.c
void init_scripting();
bool scripting_loadfile(char *filename);
void entity_action(entity_t *entity, char *action);

//text.c
void print_letter(char thechar, unsigned short x, unsigned short y);
void print_letter_h(char thechar, unsigned short x, unsigned short y);
void clear_text_box();
void text_up();
void do_say(char *string);
void say_line(char *string, unsigned short x, unsigned short y);

//savegame.c
void savegame_scripts(int slot);
void loadgame_scripts(int slot);
void do_savegame(int slot);
void do_loadgame(int slot);
void savegame();
bool loadgame();

//menu.c
void mainmenu();
unsigned char menu(char *text, unsigned short x, unsigned short y);
unsigned char do_menu(char *text, unsigned short x, unsigned short y);

//scripting.c
void do_talk();
void scripting_addfunctions();

//scripting
int character_choose(lua_State* luaS);

