#include "SDL.h"
#include "SDL_image.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <malloc.h>

#include "fcgame.h"

void init(){
	if(SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
		exit(1);
	}
	
	atexit(SDL_Quit);
	
	SDL_WM_SetIcon(IMG_Load("gfx/icon.png"), NULL);
	
	SDL_WM_SetCaption("Frikken Chicken Game", NULL);
	
	screen = SDL_SetVideoMode(640, 480, 0, SDL_SWSURFACE);
	
	if (screen == NULL) {
		fprintf(stderr, "Couldn't set 640x480x8 video mode: %s\n", SDL_GetError());
		exit(1);
	}
	
	screenwidth = 34;
	screenheight = 25;
	
	SDL_Surface *temp = sbuffer = SDL_CreateRGBSurface(SDL_SWSURFACE, screenwidth*TILEWIDTH*2, screenheight*TILEHEIGHT*2, 16, rmask, gmask, bmask, amask);
	sbuffer = SDL_DisplayFormat(temp);
	
	bufferneedsdraw = malloc(((screenheight*2)*(screenwidth*2))/8);
	
	if (sbuffer == NULL) {
		fprintf(stderr, "Couldn't create screen buffer: %s\n", SDL_GetError());
		exit(1);
	}
	
	SDL_FillRect(sbuffer, NULL, SDL_MapRGB(screen->format, 0, 255, 0));
	
	/////font
	
	font = image_load("gfx/font-black.png");
	highfont = image_load("gfx/font-white.png");

	if(font == NULL || highfont == NULL){
		exit(1);
	}
	
	init_audio();
	
	load_sounds();
	
	set_bgmusic("audio/absorbancygood.ogg");
}

void gameloop(){
	SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
	
	bool left = false;
	bool right = false;
	bool up = false;
	bool down = false;
	
	drawscreen();
	SDL_Flip(screen);
	
	init_time();
	clear_input();
	
	while(!quit){
		SDL_Event event;
		
		while(SDL_PollEvent(&event)){
			switch(event.type){
				case SDL_KEYDOWN :
					switch(event.key.keysym.sym){
						case SDLK_q :
							quit = true;
							break;
						case SDLK_ESCAPE :
							left = right = down = up = false;
							switch(menu("continue\nsave\nload\nquit", 14*7, (screen->h)-(18*12))){
								case 4 : return;
								case 0 : break;
								case 1 : break;
									break;
								case 2 :
									savegame();
									break;
								case 3:
									loadgame();
									break;
								default:
									break;
							}
							break;
						case SDLK_UP :
							up = true;
							break;
						case SDLK_DOWN :
							down = true;
							break;
						case SDLK_LEFT :
							left = true;
							break;
						case SDLK_RIGHT :
							right = true;
							break;
						case SDLK_RETURN:
						case SDLK_SPACE:
							if((player->x)*TILEWIDTH == player->realx && (player->y)*TILEHEIGHT == player->realy){
								left = right = down = up = false;
								do_talk();
							}
							break;
						default:
							break;
					}
					break;
					
				case SDL_KEYUP :
					switch(event.key.keysym.sym){
						case SDLK_UP :
							up = false;
							break;
						case SDLK_DOWN :
							down = false;
							break;
						case SDLK_LEFT :
							left = false;
							break;
						case SDLK_RIGHT :
							right = false;
							break;
						default:
							break;
					}
					break;

					
				case SDL_QUIT: {
					quit = true;
				}
			}
		}
		
		entity_move(player, ((signed short)right - (signed short)left), ((signed short)down - (signed short)up));
		
		
		refresh_entities();
		
		move_screen(player->realx - (screenwidth/2)*TILEWIDTH, (player->realy - (screenheight/2)*TILEHEIGHT));
		
		update();
	}
	
	unloadmap();
}

int main(int argc, char **argv){
	init();
	
	mainmenu();
	
	return 0;
}
