-- setup math.random --
math.randomseed(os.time())
math.random(); math.random(); math.random(); math.random()
----------------

function printf(...)
	print(string.format(unpack(arg)))
end

----------------
function getfield(f)
	local v = _G
	for w in string.gfind(f, "[%w_]+") do
		v = v[w]
	end
	return v
end

function setfield(f, v)
	local t = _G
	for w, d in string.gfind(f, "([%w_]+)(.?)") do
		if d == "." then
			t[w] = t[w] or {}
			t = t[w]
		else
			t[w] = v
		end
	end
end

------save------
function load(savename)
	infile, err = io.open(savename, "rb")
	if infile == nil then
		error("While opening: " .. (err or "no error"))
	end
	
	buf, err = infile:read("*a")
	if buf == nil then
		error("While reading: " .. (err or "no error"))
	end
	
	infile:close()
	
	p = pluto.unpersist(saveperms, buf)
	
	unpersist_entities()
	
	setup_player(p.player.x, p.player.y, p.player.direction)
	set_whatplayer(p.whatplayer)
end

function save(savename, playerx, playery, playerdirection)
	p.whatplayer = get_whatplayer()
	p.player = {x = playerx, y = playery, direction = playerdirection}
	
	persist_entities()
	
	saveperms = {}
	
	buf = pluto.persist(saveperms, p)
	
	outfile = io.open(savename, "wb")
	outfile:write(buf)
	outfile:close()
end
----------------

entities = {}
Entity_defaultflags = 0
Entity = {}
mt = {}

function Entity:new(n, xval, yval, tid, dir, f)
	_G[n] = {name = n or '', tileid = tid, x = xval, y = yval, direction = dir, flags = f}
	setmetatable(_G[n], mt)
	entities[n] = _G[n]
end

function Entity:dump()
	printf("==============================")
	printf("Name:     %s", self.name)
	printf("x:        %u", self.x)
	printf("y:        %u", self.y)
	printf("flags:    %u", self.flags)
	printf("==============================")
end

function Entity:setdirection(direc)
	self.direction = direc
end

function Entity:persist()
	p[self.name] = _G[self.name]
end

function Entity:unpersist()
	_G[self.name] = p[self.name]
end

mt.__add = function (a,b) error("ERROR: Cannot add entities!") return 0 end
mt.__mul = function (a,b) error("ERROR: Cannot multiply entities!") return 0 end
mt.__index = Entity -- redirect queries to the Entity table

----------------
left = 3
right = 1
up = 0
down = 2

function reload_entities()
	for ent in entities do
		printf("Reloading:  %s", _G[ent].name)
		create_entity(_G[ent].name, _G[ent].x, _G[ent].y, _G[ent].tileid, _G[ent].direction, _G[ent].flags)	--c function
	end
end

function persist_entities()
	p.entities = entities
	for ent in entities do
		printf("Persisting:  %s", _G[ent].name)
		_G[ent]:persist()
	end
end

function unpersist_entities()
	entities = p.entities
	for ent in entities do
		printf("Unpersisting:  %s", _G[ent].name)
		_G[ent]:unpersist()
	end
end


