#include <stdbool.h>

#include "SDL.h"
#include "SDL_audio.h"
#include "SDL_mixer.h"

#include "sounds.h"

bool sound_enabled = true;

void init_audio(){
	if(!sound_enabled) return;
	if (Mix_OpenAudio(22050, MIX_DEFAULT_FORMAT, 2, 1024) < 0) {
		fprintf(stderr, "Unable to open audio: %s\n", SDL_GetError());
 		sound_enabled = false;
		return;
	}
}

Mix_Chunk *load_sound(char *filename, int volume){
	if(!sound_enabled) return NULL;
	
	Mix_Chunk *tmp;
	tmp = Mix_LoadWAV(filename);
	if(!tmp) {
		printf("Mix_LoadWAV: %s\n", Mix_GetError());
	}
	Mix_VolumeChunk(tmp, volume);
	return tmp;
}

void load_sounds(){
	if(!sound_enabled) return;
	Mix_AllocateChannels(1);
	noise_beep = load_sound("audio/beep.wav", 20);
}

void beep(){
	if(!sound_enabled) return;
	Mix_AllocateChannels(Mix_Playing(-1)+1);
	
	if(Mix_PlayChannel(-1, noise_beep, 0) == -1){
		printf("Mix_PlayChannel: %s\n", Mix_GetError());
	}
}

void set_bgmusic(char *file){
	if(!sound_enabled) return;
	
	if(bgmusic != NULL){
		while(!Mix_FadeOutMusic(3000) && Mix_PlayingMusic()) {
    		SDL_Delay(100);
		}
		Mix_FreeMusic(bgmusic);
		bgmusic=NULL;
	}
	if(file != NULL && *file != 0){
		bgmusic=Mix_LoadMUS(file);
		if(!bgmusic) {
			printf("Mix_LoadMUS(\"%s\"): %s\n", file, Mix_GetError());
			return;
		}
		
		/*if(Mix_PlayMusic(bgmusic, -1) == -1){
			printf("Mix_PlayMusic: %s\n", Mix_GetError());
		}*/
		if(Mix_FadeInMusic(bgmusic, -1, 2000)==-1) {
			printf("Mix_FadeInMusic: %s\n", Mix_GetError());
		}
		
		printf("volume was %d, is now %d\n", Mix_VolumeMusic(MIX_MAX_VOLUME/2), Mix_VolumeMusic(-1));
	}
}
