#include <stdbool.h>

#include "SDL.h"
#include "SDL_image.h"

#include "fcgame.h"

static unsigned long next_time;

void init_time(){
	frameskip = false;
	next_time = SDL_GetTicks() + TICK_INTERVAL;
}

signed short stupiddiv(signed short value){
	if(value < 0){
		return value/TILEWIDTH - 1;
	}else{
		return value/TILEWIDTH;
	}
}

bool checkbounds(signed short x, signed short y){
	if(x < 0 || y < 0){
		return false;
	}

	if(x >= mapwidth || y >= mapheight){
		return false;
	}
	return true;
}

unsigned long time_left(){
	unsigned long now;
	
	now = SDL_GetTicks();
	if(next_time <= now){
		frameskip = true;
		return 0;
	}else{
		frameskip = false;
		return next_time - now;
	}
}

void delay(){
	SDL_Delay(time_left());
	next_time += TICK_INTERVAL;
}

void clear_input(){
	SDL_Event event;
	while(SDL_PollEvent(&event));
}

unsigned short FPS;
unsigned char frames;
unsigned long ticks;

void calculate_FPS(){
	frames++;
	if(SDL_GetTicks() >= ticks + 1000){
		//printf("%d fps\n", frames);
		FPS = frames;
		frames = 0;
		ticks = SDL_GetTicks();
	}
}

SDL_Surface *image_load(char *filename){
	SDL_Surface *tmp = IMG_Load(filename);
	if (tmp == NULL) {
		fprintf(stderr, "Couldn't load %s: %s\n", filename, SDL_GetError());
		return NULL;
	}

	SDL_Surface *retval = SDL_DisplayFormatAlpha(tmp);
	
	if (retval == NULL) {
		fprintf(stderr, "Error converting surface... (%s)\n", SDL_GetError());
		return NULL;
	}
	
	SDL_FreeSurface(tmp);
	
	return retval;
}

